package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.Brand;
import com.zh.project0512.service.IBrandService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@RestController
@RequestMapping("/manage/brand")
@Tag(name = "品牌")
public class BrandController {
    @Autowired
    private IBrandService brandService;

    @Data
    static class AddBParam {
        @NotNull(message = "name不能为空")
        @Schema(title = "品牌名称")
        private String name;
    }

    @Operation(summary = "新增品牌")
    @PostMapping("/add")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "120")
    public Result add(@Validated @RequestBody AddBParam param) {
        brandService.addBrand(new Brand().setName(param.getName()).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelBParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "品牌id")
        private Integer id;
    }

    @Operation(summary = "删除品牌(软删除)")
    @PostMapping("/del")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "121")
    public Result del(@Validated @RequestBody DelBParam param) {
        return MybatisPlusUtil.sqlResult(brandService.update(new UpdateWrapper<Brand>().eq("id", param.getId()).set("sortWeight",0).set("isDeleted", 1)), "删除");
    }

    @Data
    static class UpdBParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "品牌id")
        private Integer id;
        @NotNull(message = "name不能为空")
        @Schema(title = "品牌名称")
        private String name;
    }

    @Operation(summary = "更新品牌")
    @PostMapping("/upd")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "131")
    public Result upd(@Validated @RequestBody UpdBParam param) {
        return MybatisPlusUtil.sqlResult(brandService.updateById(new Brand().setId(param.getId()).setName(param.getName()).setUpdateAt(LocalDateTime.now())), "修改");
    }

    @Operation(summary = "品牌列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@RequestBody(required = false) @Parameter(hidden = true) JSONObject obj) {
        QueryWrapper<Brand> qw = new QueryWrapper<>();
        qw.eq("isDeleted",0).orderByAsc("sortWeight").orderByDesc("updateAt","creatAt");
        return Result.success(brandService.pageMaps(MybatisPlusUtil.SetPage(obj),qw));
    }

    @Data
    static class UpdBSParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "品牌id")
        private Integer id;
        @NotNull(message = "排序权重不能为空")
        @Min(value = 1, message = "排序权重最小值为1")
        @Schema(title = "排序权重:数字越小排序靠前")
        private Integer sortWeight;
    }

    @Operation(summary = "修改品牌排序")
    @PostMapping("/updSort")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "132")
    public Result updSort(@Validated @RequestBody UpdBSParam param) {
        Brand brand = brandService.getById(param.getId());
        if(brand.getIsDeleted() == 1){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"该品牌已被删除");
        }
        int origin = brand.getSortWeight();
        int current = param.getSortWeight();
        if(origin==current){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"权重无变化");
        }
        brandService.updSort(origin,current,origin>current);
        return Result.success("修改完成");
    }
}
